package grailsgtd

import org.springframework.dao.DataIntegrityViolationException

/**
 * Controller pro komentáře
 * U každé akce je provedene autorizace požadavku pro danou akci
 */

class CommentController {

    /* nastavení povolených http metod pro některé akce */
    static allowedMethods = [save: "POST"]

    /**
     * Uložení komentáře k úkolu určeném parametrem s následným přesměrováním na detail daného úkolu
     * @param id
     * @return
     */
    def save(Long id) {

        def task = Task.findById(id)
        
        if ((!task)||(task.user.id != session.user.id)) {
            flash.message = message(code: 'default.unauthorized')
            redirect(controller: "task", action: "list")
            return
        }

        def commentInstance = new Comment(params)
        commentInstance.task = task

        if (!commentInstance.save(flush: true)) {
            flash.message = message(code: 'comment.create.fail')
            redirect(controller: "task", action: "show", id: id)
            return
        }

        flash.message = message(code: 'comment.created.message')
        redirect(controller: "task", action: "show", id: id)
    }

    /**
     * Smazání komentáře s následným přesměrováním na detail daného úkolu
     * @param id
     * @return
     */
    def delete(Long id) {
        def commentInstance = Comment.get(id)
        if (!commentInstance || commentInstance.task.user.username != session.user.username) {
            flash.message = message(code: 'comment.not.found.message')
            redirect(controller: "task", action: "list")
            return
        }

        def taskId = commentInstance.task.id

        try {
            commentInstance.delete(flush: true)
            flash.message = message(code: 'comment.deleted.message')
            redirect(controller: "task", action: "show", id: taskId)
        }
        catch (DataIntegrityViolationException e) {
            flash.message = message(code: 'comment.not.deleted.message')
            redirect(controller: "task", action: "show", id: taskId)
        }
    }
}
