package grailsgtd

import org.springframework.dao.DataIntegrityViolationException

/**
 * Controller pro tagy
 */
class TagController {

    /* nastavení povolených http metod pro některé akce */
    static allowedMethods = [save: "POST", update: "POST"]

    /**
     * Vykresluje formulář pro vytvoření nového tagu
     * @return
     */
    def create() {
        [tagInstance: new Tag(params)]
    }

    /**
     * Provádí samotné vytvoření tagu
     * @return
     */
    def save() {
        def tagInstance = new Tag(params)
        tagInstance.user = session.user
        if (!tagInstance.save(flush: true)) {
            render(view: "create", model: [tagInstance: tagInstance])
            return
        }

        flash.message = message(code: 'tag.created.message')
        redirect(controller: "task", action: "list")
    }

    /**
     * Dle parametru načte tag z databáze a jeho hodnoty vloží do formuláře
     * pro následnou úpravu
     * @param id
     * @return
     */
    def edit(Long id) {
        def tagInstance = Tag.get(id)
        if ((!tagInstance) || (tagInstance.user.username != session.user?.username)) {
            flash.message = message(code: 'tag.not.found.message')
            redirect(controller:"task",action: "list")
            return
        }

        [tagInstance: tagInstance]
    }

    /**
     * Dle parametru provede aktualizaci tagu v databázi s kontrolou,
     * zda již mezitím nedošlo k jiné aktualizaci (druhý parametr Version)
     * @param id
     * @param version
     * @return
     */
    def update(Long id, Long version) {
        def tagInstance = Tag.get(id)
        if ((!tagInstance) || (tagInstance.user.username != session.user?.username)) {
            flash.message = message(code: 'tag.not.found.message')
            redirect(controller: "task", action: "list")
            return
        }

        if (version != null) {
            if (tagInstance.version > version) {
                tagInstance.errors.rejectValue("version", "default.optimistic.locking.failure")
                render(view: "edit", model: [tagInstance: tagInstance])
                return
            }
        }

        tagInstance.properties = params

        if (!tagInstance.save(flush: true)) {
            render(view: "edit", model: [tagInstance: tagInstance])
            return
        }

        flash.message = message(code: 'tag.updated.message')
        redirect(controller: "task", action: "list")
    }

    /**
     * Na základě parametru provede smazání záznamu
     * @param id
     * @return
     */
    def delete(Long id) {
        def tagInstance = Tag.get(id)
        if ((!tagInstance) || (tagInstance.user.username != session.user?.username)) {
            flash.message = message(code: 'tag.not.found.message')
            redirect(controller: "task", action: "list")
            return
        }

        try {
            tagInstance.delete(flush: true)
            flash.message = message(code: 'tag.deleted.message')
        }
        catch (DataIntegrityViolationException e) {
            flash.message = message(code: 'tag.not.deleted.message')
        }

        redirect(controller: "task", action: "list")
    }
}
