package grailsgtd

import static org.junit.Assert.*

import grails.test.mixin.*
import grails.test.mixin.support.*
import org.junit.*

/**
* Testování controlleru a modelu komentářů
*/
@TestMixin(GrailsUnitTestMixin)
@TestFor(CommentController)
@Mock([Comment, Task, State, User])
class CommentControllerTests {

    private user

    /**
     * Přípravná fáze, definování přihlášeného uživatele
     */
    void setUp() {
        
        this.user = fixtureUser()
        
        controller.session.user = this.user

    } 

    /**
     * Textování reakce na špatně vyplněný přidávací formulář
     */
    void testAddFail() {
        def id = fixtureTask()
        
        controller.save(id)

        assert response.redirectedUrl == '/task/show/' + id
        assert Comment.count() == 0
    }

    /**
     * Testování reakce na pokus o přidání komentáře k úkolu, který nevlastní daný uživatel
     */
    void testAddNotAuthorized() {

        def id = fixtureTask()
        
        params.text = ""
        controller.session.user = fixtureFailUser()
        controller.save(id)

        assert response.redirectedUrl == '/task/list'
        assert Comment.count() == 0
    }

    /**
     * Testování funkčnosti přidávacího formuláře
     */
    void testAdd() {
        def id = fixtureTask()
        
        params.text = "Text"
        controller.save(id)

        assert response.redirectedUrl == '/task/show/' + id
        assert Comment.count() == 1
    }

    /**
     * Testování reakce při pokusu smazat komentář, jehož úkol nevlastní daný uživatel
     */
    void testDeleteNotAuthorized(){
        def id = fixtureTask()       
        def comment = new Comment(text: 'pokus', task: Task.findById(id))
        comment.save()
        
        controller.session.user = fixtureFailUser()        
        controller.delete(comment.id)
        
        assert response.redirectedUrl == '/task/list'        
        assert Comment.count() == 1        
    }

    /**
     * Testování funkčnosti mechanismu pro smazání komentáře
     */
    void testDelete() {
        def id = fixtureTask()       
        def comment = new Comment(text: 'pokus', task: Task.findById(id))
        comment.save()
        
        controller.delete(comment.id)
        
        assert response.redirectedUrl == '/task/show/' + id
        assert Comment.count() == 0
    }

    /**
     * Vytvoření testovacího úkolu
     * @return
     */
    private fixtureTask() {
        def t = new Task(
            name: "pokus",
            description: "pokus",
            todate: new Date(),
            user: this.user,
            priority: 1,
            state: fixtureState() 
        )

        t.save()
        
        return t.id
    }

    /**
     * Vytvoření testovacího uživatele
     * @return
     */
    private fixtureUser(){
        def u =  new User(username: "pokus@pokus.cz", passwordRegistration: "pokus", passwordConfirm: "pokus")
        u.save()
        
        return u 
    }

    /**
     * Vytvoření testovacího stavu
     * @return
     */
    private fixtureState(){
        def s = new State(name: "pokus")
        s.save()
        
        return s
    }

    /**
     * Vytvoření fiktivního uživatele
     * @return
     */
    private fixtureFailUser(){
        def u =  new User(username: "pokusny@pokus.cz", passwordRegistration: "pokus", passwordConfirm: "pokus")
        u.save()
        
        return u    
    }
}
