package grailsgtd

import static org.junit.Assert.*

import grails.test.mixin.*
import grails.test.mixin.support.*
import org.junit.*

/**
* Testování controlleru a modelu tagů
*/
@TestMixin(GrailsUnitTestMixin)
@TestFor(TagController)
@Mock([Tag, User, Task, State, Comment])
class TagControllerTests {

    /**
     * Přípravná fáze, vytvoření uživatele a jeho "přihlášení"
     */
    void setUp() {

        User u = new User(
                username: "pokus@pokus.cz",
                passwordConfirm: "pokus",
                passwordRegistration: "pokus"
        )

        controller.session.user = u

    }

    /**
     * Pokus o vytvoření tagu s neplatným atributem
     */
    void testCreateFail() {
        params.name = ""

        controller.save()

        assert model.tagInstance != null
        assert view == '/tag/create'
    }

    /**
     * Testování formuláře pro vytvoření tagu
     */
    void testCreate() {
        params.name = "pokus"

        controller.save()

        assert model.tagInstance == null
        assert response.redirectedUrl == '/task/list'

        assert Tag.count() == 1
    }

    /**
     * Testování reakce při pokusu editovat cizí tag
     */
    void testEditUpdateNotAutthorized() {

        def id = fixture()

        session.invalidate()
        controller.edit(id)

        assert response.redirectedUrl == '/task/list'
    }

    /**
     * Testování funkčnosti formuláře pro aktualizaci tagu
     * !!! Nefunkční update, zřejmě bug, reportováno a evidováno pod identifikátorem GRAILS-9885
     */
    void testUpdate() {
        def id = fixture()

        def name = "pokus1"

        params.id = 0
        params.name = name
        params.version = 0

        //controller.update(id, 0)

        //assert response.redirectedUrl == '/task/list'
        //assert Tag.get(id).name == name
    }

    /**
     * Testování reakce při pokusu smazat cizí tag
     */
    void testDeleteNotAuthorized() {
        def id = fixture()

        session.invalidate()
        controller.delete(id)

        assert response.redirectedUrl == '/task/list'
        assert Tag.count() == 1
    }

    /**
     * Testování funkčnosti smazání tagu
     */
    void testDelete() {
        def id = fixture()

        controller.delete(id)

        assert response.redirectedUrl == '/task/list'
        assert Tag.count() == 0
    }

    /**
     * Vytvoření testováho tagu
     * @return
     */
    private int fixture() {
        def t = new Tag(name: "pokus", user: controller.session.user)
        t.save()

        return t.id
    }
}
