package grailsgtd

import static org.junit.Assert.*

import grails.test.mixin.*
import grails.test.mixin.support.*
import org.junit.*

/**
* Testování controlleru a modelu uživatelé
*/
@TestFor(UserController)
@Mock(User)
class UserControllerTests {

    /**
     * Testování reakce při pokusu uložit špatně vyplněný registrační formulář
     */
    void testRegistrationFormInvalid() {
        params.username = "pokus"
        params.passwordRegistration = "pokus"

        controller.save()

        assert model.userInstance != null
        assert view == '/user/registration'
    }

    /**
     * Testování funkčnosti registračního formuláře
     */
    void testRegistrationFormValid(){

        params.username = "pokus@pokus.cz"
        params.passwordRegistration = "pokus"
        params.passwordConfirm = "pokus"

        controller.save()

        assert response.redirectedUrl == '/user/login'
        assert flash.message != null
        assert User.count() == 1
        assert User.get(1).password ==  params.passwordRegistration.encodeAsMD5()
    }

    /**
     * Testování funkce přihlášení
     */
    void testLogin(){
        createUser()

        params.username = "pokus@pokus.cz"
        params.password = "pokus"

        controller.doLogin()

        assert response.redirectedUrl == '/task/list'
        assert flash.message == null
        assert session.user != null
    }

    /**
     * Testování reakce při pokus o přihlášení se špatnými údaji
     */
    void testLoginFail(){
        createUser()

        params.username = "pokus@pokus.cz"
        params.password = "pokuss"

        controller.doLogin()

        assert view == '/user/login'
        assert flash.message != null
        assert session.user == null
    }

    /**
     * Vytvoření testovacího uživatele
     */
    private void createUser(){
        User u = new User(
            username: "pokus@pokus.cz",
            passwordConfirm: "pokus",
            passwordRegistration: "pokus"
        )
        u.save()
    }

    /**
     * Testování funkčnosti odhlášení
     */
    void testLogout(){
        User u = new User(
                username: "pokus@pokus.cz",
                passwordConfirm: "pokus",
                passwordRegistration: "pokus"
        )

        session.user = u

        assert session.user != null

        controller.logout()

        assert session.user == null
        assert response.redirectedUrl == '/user/login'
    }
}
