<?php
namespace Models;

use \Nette\Database\Connection;


/**
 * Základní abstraktní třída modelu
 */
abstract class Base extends \Nette\Object
{
    /** @var \Nette\Database\Connection */
    public $db;

    /** @var string */
    private $tableName;


    /**
     * @param \Nette\Database\Connection $database
     */
    public function __construct(Connection $database)
    {
        $this->tableName = $this->tableNameByClass(get_class($this));
        $this->db = $database;

    }


    /**
     * Zahájení transakce
     * @return mixed
     */
    public function begginTransaction()
    {
        return $this->db->beginTransaction();
    }


    /**
     * Commit
     * @return mixed
     */
    public function commit()
    {
        return $this->db->commit();
    }

    /**
     * Rollback
     * @return mixed
     */
    public function rollBack()
    {
        return $this->db->rollBack();
    }


    /**
     * Vrátí instanci \Nette\Database\Table\Selection dle aktuálního modelu
     * @return \Nette\Database\Table\Selection
     */
    public function getConnection()
    {
        return $this->db->table($this->tableName);
    }


    /**
     * Určí tabulku dle názvu třídy
     * @param string
     * @return string
     * @result: Pages => pages, ArticleTag => article_tag
     */
    private function tableNameByClass($className)
    {
        $tableName = explode("\\", $className);
        $tableName = lcfirst(array_pop($tableName));

        $replace = array(); // A => _a
        foreach (range("A", "Z") as $letter) {
            $replace[$letter] = "_" . strtolower($letter);
        }

        return strtr($tableName, $replace);
    }

    /**
     * Vložení dat z parametru do tabulky
     * @param null $data
     * @return \Nette\Database\Table\ActiveRow
     */
    public function insert($data)
    {
        return $this->getConnection()->insert($data);
    }

    /**
     * Aktualizace záznamu
     * @param $data
     * @return int
     */
    public function update($data)
    {
        return $this->getConnection()->update($data);
    }

    /**
     * Smazání záznamu
     * @param $id
     * @return int
     */
    public function delete($id)
    {
        return $this->getConnection()->find($id)->delete();
    }

    /**
     * Vrácení záznamu dle id
     * @param $id
     * @return \Nette\Database\Table\ActiveRow
     */
    public function fetch($id)
    {
        return $this->getConnection()->get($id);
    }

    /**
     * Vrácení celé množiny záznamů
     * @return \Nette\Database\Table\Selection
     */
    public function fetchAll()
    {
        return $this->getConnection();
    }


    /**
     * Vrácení páru klíč/hodnota dle parametru
     * @param $key
     * @param $value
     * @return array
     * @throws \Nette\InvalidArgumentException
     */
    public function fetchPairs($key, $value)
    {
        return $this->getConnection()->fetchPairs($key, $value);
    }

    /**
     * Vrácení počtu záznamů
     * @return int
     */
    public function count()
    {
        return $this->getConnection()->count();
    }
}