<?php

use \Nette\Application\UI\Form;

/**
 * Tag presenter.
 */
class TagPresenter extends BasePresenter
{

    /** @var \Models\TaskTag */
    protected $tagModel;


    /**
     * Injektáž modelů
    * @param Models\Tag $tagModel
     * @throws Nette\InvalidStateException
     */
    public function injectModels(\Models\Tag $tagModel)
    {
        if ($this->tagModel) {
            throw new Nette\InvalidStateException('Injektáž již proběhla.');
        }
        $this->tagModel = $tagModel;
    }


    /**
     * Zobrazení editace s využitím formuláře a view pro přidávání tagů
     * @param $id
     */
    public function renderEdit($id)
    {
        if (!$id) {
            $this->error('Neplatný identifikátor');
        }

        // proveď pouze pokud je formulář odeslán
        if (!$this['tagForm']->isSubmitted()) {
            $tag = $this->tagModel->fetch($id, $this->getUser()->getId());

            if(!$tag){
                $this->error('Neplatný identifikátor');
            }

            // nastavení výchozích hodnot
            $this['tagForm']->setDefaults($tag);
        }

        $this->setView('add');
    }


     /**
     * Továrna formuláře pro úpravu tagu
     * @return Nette\Application\UI\Form
     */
    protected function createComponentTagForm()
    {
        $form = new Form();

        $form->addText('name', 'Název:')
            ->setRequired('Vyplňte název tagu');

        $form->addSubmit('send', 'Uložit');

        $form->onSuccess[] = $this->tagFormSucceeded;
        return $form;
    }


    /**
     * Callback pro odeslaný formulář s tagem
     * @param $form
     */
    public function tagFormSucceeded($form)
    {
        if ($form->isSubmitted()) {
            $id = (int)$this->getParameter('id');

            $values = $form->getValues();

            // proveď update, pokud se jedná o editaci, jinak inser
            if ($tag = $this->tagModel->fetch($id, $this->getUser()->getId())) {
                $tag->update($values);
            } else {
                $values['user_email'] = $this->getUser()->getId();

                $this->tagModel->insert($values);
            }

            $this->flashMessage('Tag byl úspěšně uložen.', 'success');
            $this->redirect('Task:default');

        }
    }


    /**
     * Smaže tag dle id
     * @param $id
     */
    public function actionDelete($id)
    {
        if (!$id) {
            $this->error('Neplatný identifikátor');
        }

        if ($this->tagModel->delete($id, $this->getUser()->getId())) {
            $this->flashMessage('Tag byl úspěšně smazán.', 'success');
        } else {
            $this->flashMessage('Tag se nepodařilo smazat.', 'error');
        }

        $this->redirect('Task:default');
    }
}
