<?php

use Nette\Application\UI, \Nette\Application\UI\Form;


/**
 * User presenters.
 */
class UserPresenter extends BasePresenter
{

    /** @var \Models\User */
    protected $userModel;

    /**
     * @param Models\User $userModel
     * @throws Nette\InvalidStateException
     * @return void
     */
    public function injectModels(\Models\User $userModel)
    {
        if ($this->userModel) {
            throw new Nette\InvalidStateException('Injektáž již proběhla.');
        }

        $this->userModel = $userModel;
    }

    /**
     * Továrna formuláře pro registraci
     * @return Nette\Application\UI\Form
     */
    protected function createComponentRegForm()
    {
        $form = new UI\Form;
        $form->addText('email', 'E-Mail:')
            ->setRequired('Vyplňte váš email.')
            ->addRule(Form::EMAIL, 'Neplatný typ e-mailu.')
            ->addRule(function (\Nette\Forms\IControl $control) {
                return ($this->userModel->fetchAll()->where('email', $control->getValue())->count() > 0) ? FALSE : TRUE;
                }, 'Toto uživatelské jméno již existuje.');

        $form->addPassword('password', 'Heslo:')
            ->setRequired('Vyplňte vaše heslo.');

        $form->addPassword('password1', 'Heslo:')
            ->setRequired('Vyplňte heslo pro kontrolu.')
            ->addRule(Form::EQUAL, 'Hesla se neshodují', $form['password']);


        $form->addSubmit('send', 'Registrovat');

        $form->onSuccess[] = $this->regFormSucceeded;
        return $form;
    }

    /**
     * Callback pro odeslaný registrační formulář
     * @param $form
     */
    public function regFormSucceeded($form)
    {

        if ($form->isSubmitted()) {
            $values = $form->getValues();

            unset($values['password1']);
            $values['password'] = $this->userModel->calculateHash($values['password']);

            if ($this->userModel->insert($values)) {
                $this->flashMessage('Registrace proběhla úspěšně, můžete se přihlásit.', 'success');
                $this->redirect('in');
            } else {
                $this->flashMessage('Registrace se nezdařila, tento email zřejmě v databázi již existuje.', 'error');
            }
        }


    }

    /**
     * Továrna formuláře na přihlášení
     * @return Nette\Application\UI\Form
     */
    protected function createComponentSignInForm()
    {
        $form = new UI\Form;
        $form->addText('email', 'E-Mail:')
            ->setRequired('Vyplňte váš email.');


        $form->addPassword('password', 'Heslo:')
            ->setRequired('Vyplňte vaše heslo.');

        $form->addCheckbox('remember', 'Zapamatuj si mě');

        $form->addSubmit('login', 'Přihlásit');

        $form->onSuccess[] = $this->signInFormSucceeded;
        return $form;
    }


    /**
     * Callback pro odeslaný přihlašovací formulář
     * @param $form
     */
    public function signInFormSucceeded($form)
    {

        $values = $form->getValues();

        if ($values->remember) {
            $this->getUser()->setExpiration('+ 14 days', FALSE);
        } else {
            $this->getUser()->setExpiration('+ 20 minutes', TRUE);
        }

        try {
            $this->getUser()->login($values->email, $values->password);
        } catch (Nette\Security\AuthenticationException $e) {
            $this->flashMessage('Přihlášení se nezdařilo, bylo zadáno neplatné jméno nebo heslo.', 'error');
            return;
        }

        $this->redirect('Task:default');
    }


    /**
     * Akce pro odhlášení
     */
    public function actionOut()
    {
        $this->getUser()->logout();
        $this->flashMessage('Byl/a jste úspěšně odhlášen/a.', 'success');
        $this->redirect('in');
    }
}
