<?php

 /**
 * Formulář pro přidávání komentářů
 */
class CommentForm extends \Nette\Application\UI\Form{


    /**
     * @var \Models\Comment
     */
    private $commentModel;

    /**
     * @var \Models\Task
     */
    private $taskModel;


    /**
     * @param Models\Comment $commentModel
     * @param Models\Task $taskModel
     */
    public function __construct(\Models\Comment $commentModel, \Models\Task $taskModel){
        parent::__construct(null, null);

        $this->commentModel = $commentModel;
        $this->taskModel = $taskModel;

        $this->addTextArea('text', null)
            ->setRequired('Vyplňte text komentáře.');

        $this->addSubmit('send', 'Uložit');

        $this->onSuccess[] = $this->commentFormSucceeded;
    }


    /**
     * Obsluha formuláře pro komentáře
     * @param $form
     */
    public function commentFormSucceeded($form)
    {
        if ($form->isSubmitted()) {
            $values = $form->getValues();

            $id = (int) $this->presenter->getParameter('id');

            if(!$this->taskModel->fetch($id, $this->presenter->getUser()->getId())){
                $this->presenter->error('Neautorizovaná akce');
            }

            $values['task_id'] = $id;
            $values['date'] = new \Nette\DateTime();

            if($this->commentModel->insert($values)){
                $this->presenter->flashMessage('Komentář byl úspěšně uložen.', 'success');
            }else{
                $this->presenter->flashMessage('Komentář se nepodařilo uložit.', 'error');
            }

            $this->presenter->redirect('Task:view', array('id' => $id));
        }
    }
}