<?php

namespace Test;

use Nette,
    Tester,
    Tester\Assert;

$container = require __DIR__ . '/bootstrap.php';


/**
 * Testování CommentPresenteru v přihlášeném stavu
 */
class CommentPresenterLoggedTest extends Tester\TestCase
{
    /**
     * @var \Nette\DI\Container
     */
    private $container;
    /**
     * @var identifikátor testovaného záznamu
     */
    private $id;

    /**
     * @var identifikátor task fixture testovaného záznamu
     */
    private $taskId;

    /**
     * @var \CommentPresenter
     */
    private $presenter;

    /**
     * @param \Nette\DI\Container $container
     */
    function __construct(Nette\DI\Container $container)
    {
        $this->container = $container;

    }

    /**
     * Přípravná fáze, vytvoření presenteru, injekce a přihlášení
     */
    function setUp()
    {
        $this->presenter = new \CommentPresenter($this->container);
        $this->presenter->injectModels($this->container->getService('commentModel'));

        $this->login();


        $this->taskId = $this->container->getService('taskModel')->insert(array(
                'name' => 'Pokus',
                'description' => 'Pokus',
                'priority' => '1',
                'state_id' => '1',
                'description' => 'Pokus',
                'todate' => '1.1.2012',
                'user_email' => 'test@test.cz'
            )
        );

        $this->id = $this->container->getService('commentModel')->insert(array(
            'text' => 'pokus',
            'task_id' => $this->taskId
        ));

    }

    /**
     * Úklid po testech
     */
    function tearDown()
    {
        $this->container->getService('taskModel')->delete($this->taskId);
    }

    /**
     * Přihlášení testovacího uživatele
     */
    private function login(){
        $request = new \Nette\Application\Request('User','POST',
            array(
                'action' => 'in',
                'do' => 'signInForm-submit'
            ),
            array(
                'email' => 'test@test.cz',
                'password' => 'test1'
            )
        );
        $this->container->createInstance('UserPresenter')->run($request);

    }


    /**
     * Přihlášení testovacího uživatele
     */
    private function loginAnother(){
        $request = new \Nette\Application\Request('User','POST',
            array(
                'action' => 'in',
                'do' => 'signInForm-submit'
            ),
            array(
                'email' => 'test1@test.cz',
                'password' => 'password'
            )
        );
        $this->container->createInstance('UserPresenter')->run($request);

    }

    /**
     * Testování smazání komentáře při neautorizovaném přístupu
     */
    public function testCommentDeleteNotAuthorized()
    {
        $this->loginAnother();

        $request = new \Nette\Application\Request('Comment', 'POST',
            array(
                'action' => 'delete',
                'id' => $this->id
            )
        );

        try{
            $this->presenter->run($request);
        }catch (\Exception $e){
        }

        Assert::true($e instanceof Nette\Application\BadRequestException);
    }

    /**
     * Testování smazání komentáře
     */
    public function testCommentDelete()
    {
        $id = $this->container->getService('commentModel')->fetchAll()->order('id DESC')->fetch()->id;

        $request = new \Nette\Application\Request('Comment', 'POST',
            array(
                'action' => 'delete',
                'id' => $id
            )
        );

        $response = $this->presenter->run($request);
        Assert::true($response instanceof \Nette\Application\Responses\RedirectResponse);

        Assert::false($this->container->getService('commentModel')->fetch($this->id));
    }

}


id(new CommentPresenterLoggedTest($container))->run();