<?php

namespace Test;

use Nette,
    Tester,
    Tester\Assert;

$container = require __DIR__ . '/bootstrap.php';


/**
 * Testování TagPresenteru v přihlášeném stavu
 */
class TagPresenterLoggedTest extends Tester\TestCase
{
    /**
     * @var \Nette\DI\Container
     */
    private $container;
    /**
     * @var identifikátor testovaného záznamu
     */
    private $id;

    /**
     * @var \TagPresenter
     */
    private $presenter;

    /**
     * @param \Nette\DI\Container $container
     */
    function __construct(Nette\DI\Container $container)
    {
        $this->container = $container;
    }


    /**
     * Přípravná fáze, vytvoření presenteru, injekce a přihlášení
     */
    function setUp()
    {
        $this->presenter = new \TagPresenter($this->container);
        $this->presenter->injectModels(
            $this->container->getService('tagModel')
        );

        $this->login();
    }

    /**
     * Přihlášení testovacího uživatele
     */
    private function login(){
        $request = new \Nette\Application\Request('User','POST',
            array(
                'action' => 'in',
                'do' => 'signInForm-submit'
            ),
            array(
                'email' => 'test@test.cz',
                'password' => 'test1'
            )
        );
        $this->container->createInstance('UserPresenter')->run($request);

    }


    /**
     * Přihlášení testovacího uživatele
     */
    private function loginAnother(){
        $request = new \Nette\Application\Request('User','POST',
            array(
                'action' => 'in',
                'do' => 'signInForm-submit'
            ),
            array(
                'email' => 'test1@test.cz',
                'password' => 'password'
            )
        );
        $this->container->createInstance('UserPresenter')->run($request);

    }


    /**
     * Testování přítomnosti formuláře pro přidání úkolu
     */
    public function testAddTagForm(){
        $request = new \Nette\Application\Request('Tag','POST',
            array(
                'action' => 'add',
            )
        );

        $response = $this->presenter->run($request);

        Assert::true($response->getSource()->presenter['tagForm'] instanceof \Nette\Application\UI\Form);
    }



    public function testAdd(){
        $request = new \Nette\Application\Request('Tag','POST',
            array(
                'action' => 'add',
                'do' => 'tagForm-submit'
            ),
            array(
                'name' => 'Pokus',
            )
        );

        $response = $this->presenter->run($request);

        $this->id = $this->container->getService('tagModel')->fetchAll()->order('id DESC')->fetch()->id;

        Assert::true($response instanceof \Nette\Application\Responses\RedirectResponse);
    }

    public function testAddFailure(){
        $request = new \Nette\Application\Request('Tag','POST',
            array(
                'action' => 'add',
                'do' => 'tagForm-submit'
            ),
            array(
                'name' => '',
            )
        );

        $response = $this->presenter->run($request);

        Assert::true($response instanceof \Nette\Application\Responses\TextResponse);
    }


    /**
     * Testování přítomnosti formuláře pro přidání tagu
     */
    public function testEditTagForm(){
        $request = new \Nette\Application\Request('Tag','POST',
            array(
                'action' => 'edit',
                'id' => $this->id
            )
        );

        $response = $this->presenter->run($request);

        Assert::true($response->getSource()->presenter['tagForm'] instanceof \Nette\Application\UI\Form);
    }


    /**
     * Testování přítomnosti formuláře pro přidání tagu při neautorizovaném přístupu
     */
    public function testEditTagFormNotAuthorized(){
        $this->loginAnother();

        $request = new \Nette\Application\Request('Tag','POST',
            array(
                'action' => 'edit',
                'id' => $this->id
            )
        );

        try{
            $this->presenter->run($request);
        }catch (\Exception $e){
        }

        Assert::true($e instanceof Nette\Application\BadRequestException);
    }


    public function testEdit(){
        $request = new \Nette\Application\Request('Tag','POST',
            array(
                'action' => 'edit',
                'do' => 'tagForm-submit',
                'id' => $this->id
            ),
            array(
                'name' => 'Pokus',
            )
        );

        $response = $this->presenter->run($request);

        Assert::true($response instanceof \Nette\Application\Responses\RedirectResponse);
    }

    public function testEditFailure(){
        $request = new \Nette\Application\Request('Tag','POST',
            array(
                'action' => 'edit',
                'do' => 'tagForm-submit',
                'id' => $this->id
            ),
            array(
                'name' => '',
            )
        );

        $response = $this->presenter->run($request);

        Assert::true($response instanceof \Nette\Application\Responses\TextResponse);
    }

    /**
     * Testování signálu pro smazání tagu
     */
    public function testDelete(){
        $request = new \Nette\Application\Request('Tag','POST',
            array(
                'action' => 'delete',
                'id' => $this->id
            )
        );

        $response = $this->presenter->run($request);
        Assert::true($response instanceof \Nette\Application\Responses\RedirectResponse);

        Assert::false($this->container->getService('tagModel')->fetch($this->id));
    }
}


id(new TagPresenterLoggedTest($container))->run();