<?php

namespace Test;

use Nette,
    Tester,
    Tester\Assert;

$container = require __DIR__ . '/bootstrap.php';


/**
 * Testování task presenteru v nepřihlášeném stavu
 */
class TaskPresnterNotLoggedTest extends Tester\TestCase
{
    /**
     * @var \Nette\DI\Container
     */
    private $container;

    /**
     * @var \TaskPresenter
     */
    private $presenter;

    /**
     * @param \Nette\DI\Container $container
     */
    function __construct(Nette\DI\Container $container)
    {
        $this->container = $container;
    }


    /**
     * Přípravná fáze, vytvoření presenteru
     */
    function setUp()
    {
        $this->presenter = new \TaskPresenter($this->container);
    }


    /**
     * Testování zda dojde k přesměrování na přihlašovací formulář
     * při pokusu zobrazit některou z privátních akcí
     */
    function testAuthentificationRedirect()
    {
        $request = new \Nette\Application\Request('Task','POST',
            array('action' => 'detail')
        );
        $response = $this->presenter->run($request);

        Assert::contains('user/in', $response->getUrl());



        $request = new \Nette\Application\Request('Task','POST',
            array('action' => 'edit')
        );
        $response = $this->presenter->run($request);

        Assert::contains('user/in', $response->getUrl());


        $request = new \Nette\Application\Request('Task','POST',
            array('action' => 'add')
        );
        $response = $this->presenter->run($request);

        Assert::contains('user/in', $response->getUrl());



        $request = new \Nette\Application\Request('Task','POST',
            array(
                'action' => 'default',
                'do' => 'Delete'
            )
        );
        $response = $this->presenter->run($request);

        Assert::contains('user/in', $response->getUrl());



        $request = new \Nette\Application\Request('Task','POST',
            array(
                'action' => 'default',
                'do' => 'selectTag'
            )
        );
        $response = $this->presenter->run($request);

        Assert::contains('user/in', $response->getUrl());


        $request = new \Nette\Application\Request('Task','POST',
            array(
                'action' => 'default',
                'do' => 'deselectTag'
            )
        );
        $response = $this->presenter->run($request);

        Assert::contains('user/in', $response->getUrl());

        $request = new \Nette\Application\Request('Task','POST',
            array(
                'action' => 'default',
                'do' => 'delete'
            )
        );
        $response = $this->presenter->run($request);

        Assert::contains('user/in', $response->getUrl());


        $request = new \Nette\Application\Request('Task','POST',
            array(
                'action' => 'default',
                'do' => 'DeleteSearchParams'
            )
        );
        $response = $this->presenter->run($request);

        Assert::contains('user/in', $response->getUrl());
    }
}


id(new TaskPresnterNotLoggedTest($container))->run();