<?php

namespace Test;

use Nette,
    Tester,
    Tester\Assert;

$container = require __DIR__ . '/bootstrap.php';


/**
 * Testování presenteru user
 */
class UserPresenterTest extends Tester\TestCase
{
    /**
     * @var \Nette\DI\Container
     */
    private $container;

    /**
     * @var \UserPresenter
     */
    private $presenter;

    /**
     * Přihlašovací údaje
     * @var array
     */
    private $credentials = array(
        'email' => 'test@test.cz',
        'password' => 'test1',
        'password1' => 'test1'
    );


    /**
     * @param \Nette\DI\Container $container
     */
    function __construct(Nette\DI\Container $container)
    {
        $this->container = $container;
    }


    /**
     * Přípravná fáze, vytvoření presenteru a injekce
     */
    function setUp()
    {
        $this->presenter = new \UserPresenter($this->container);
        $this->presenter->injectModels($this->container->getService('userModel'));
    }


    /**
     * Testování přítomnosti přihlašovacího formuláře
     */
    function testLoginForm()
    {
        $request = new \Nette\Application\Request('User','POST', array('action' => 'in'));

        $response = $this->presenter->run($request);

        Assert::true(
            $response->getSource()->presenter['signInForm']
                instanceof
                \Nette\Application\UI\Form
        );
    }

    /**
     * Testování reakce na neplatné hodnoty
     */
    function testLoginSubmitFail(){
        $request = new \Nette\Application\Request('User','POST',
            array(
                'action' => 'in',
                'do' => 'signInForm-submit'
            ),
            array(
                'email' => 'test@pokus.cz',
                'password' => 'testovaci'
            )
        );

        $response = $this->presenter->run($request);

        Assert::true(
            $response instanceof \Nette\Application\Responses\TextResponse
        );
    }


    /**
     * Testování přítomnosti registračního formuláře
     */
    function testRegForm(){
        $request = new \Nette\Application\Request('User','POST',
            array(
                'action' => 'reg',
            )
        );

        $response = $this->presenter->run($request);
        Assert::true(
            $response->getSource()->presenter['regForm']
                instanceof
                \Nette\Application\UI\Form
        );
    }

    /**
     * Testování reakce registračního formuláře na vyplnění neplatných hodnot
     */
    function testRegInvalidInput(){

        $request = new \Nette\Application\Request('User','POST',
            array(
                'action' => 'reg',
                'do' => 'regForm-submit'
            ),
            array(
                'email' => 'test',
                'password' => 'test1',
                'password1' => 'test2'
            )
        );

        $response = $this->presenter->run($request);

        Assert::true(
            $response instanceof \Nette\Application\Responses\TextResponse
        );
    }

    /**
     * Testování reakce registračního formuláře na vyplnění platných hodnot
     */
    function testRegValidInput(){

        $request = new \Nette\Application\Request('User','POST',
            array(
                'action' => 'reg',
                'do' => 'regForm-submit'
            ),
            $this->credentials
        );

        $response = $this->presenter->run($request);

        // opačný postup, předpokladem je existence testovacího uživatele v databázi,
        // testuje se, zda formulář vrátil chybu
        Assert::true(
            $response instanceof \Nette\Application\Responses\TextResponse
        );
    }



    /**
     * Testování reakce přihlašovacího formuláře na vyplnění platných údajů
     */
    function testLoginSubmit(){
        $request = new \Nette\Application\Request('User','POST',
            array(
                'action' => 'in',
                'do' => 'signInForm-submit'
            ),
           $this->credentials

        );

        $response = $this->presenter->run($request);

        Assert::true(
            $response instanceof \Nette\Application\Responses\RedirectResponse
        );
    }

}

id(new UserPresenterTest($container))->run();